@@ -1,9 +1,9 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
+# pylint: disable=line-too-long
 """A series of tests that access content outside of this repository."""
 
-import sys
 import textwrap
 import unittest
 
@@ -17,7 +17,7 @@ def _skip_from_ssl_error(url):
     """bool: Check if the given URL can be reached."""
     # This function is mostly meant for pypy3
     try:
-        from _cffi_ssl._stdssl import error
+        from _cffi_ssl._stdssl import error  # pylint: disable=import-outside-toplevel
     except ImportError:
         return False
 
@@ -27,19 +27,19 @@ def _skip_from_ssl_error(url):
         return True
 
 
-class Reader(unittest.TestCase):
+class SourceReader(unittest.TestCase):
     """Check that external queries work."""
 
     @mock.patch("code_include.source_code._get_app_inventory")
     def _test_import(self, content, expected, _get_app_inventory):
-        """A generic test function. It tests for some source code from an importable Python object.
+        """Test for some source code from an importable Python object.
 
         Args:
             content (list[str]):
                 The lines that the user provides in a standard code-include block.
             expected (str):
                 The converted source-code text that will be tested for.
-            _get_app_inventory (:class:`mock.mock.MagicMock`):
+            _get_app_inventory (mock.mock.MagicMock):
                 The function that's normally used to query a Sphinx
                 project's inventory to find every HTML file-path and
                 tag-able header.
@@ -53,7 +53,6 @@ class Reader(unittest.TestCase):
         self.assertEqual(1, len(nodes))
         self.assertEqual(expected, nodes[0].astext())
 
-    # TODO : Add skip if the URL isn't readable
     @mock.patch("code_include.source_code._get_source_module_data")
     @mock.patch("code_include.source_code._get_source_code_from_object")
     @mock.patch("code_include.source_code._get_app_inventory")
@@ -246,90 +245,81 @@ class Reader(unittest.TestCase):
 
     def test_import(self):
         """Get the source-code of an importable object."""
-        version = sys.version_info
+        expected = textwrap.dedent(
+            '''\
+            def fill(text, width=70, **kwargs):
+                """Fill a single paragraph of text, returning a new string.
 
-        if version.major < 3:
-            expected = textwrap.dedent(
-                '''\
-                def join(a, *p):
-                    """Join two or more pathname components, inserting '/' as needed.
-                    If any component is an absolute path, all previous path components
-                    will be discarded.  An empty last part will result in a path that
-                    ends with a separator."""
-                    path = a
-                    for b in p:
-                        if b.startswith('/'):
-                            path = b
-                        elif path == '' or path.endswith('/'):
-                            path +=  b
-                        else:
-                            path += '/' + b
-                    return path'''
-            )
-        elif version.major >= 3 and version.minor <= 5:
-            expected = textwrap.dedent(
-                '''\
-                def join(a, *p):
-                    """Join two or more pathname components, inserting '/' as needed.
-                    If any component is an absolute path, all previous path components
-                    will be discarded.  An empty last part will result in a path that
-                    ends with a separator."""
-                    sep = _get_sep(a)
-                    path = a
-                    try:
-                        if not p:
-                            path[:0] + sep  #23780: Ensure compatible data type even if p is null.
-                        for b in p:
-                            if b.startswith(sep):
-                                path = b
-                            elif not path or path.endswith(sep):
-                                path += b
-                            else:
-                                path += sep + b
-                    except (TypeError, AttributeError, BytesWarning):
-                        genericpath._check_arg_types('join', a, *p)
-                        raise
-                    return path'''
-            )
-        elif version.major >= 3 and version.minor >= 6:
-            expected = textwrap.dedent(
-                '''\
-                def join(a, *p):
-                    """Join two or more pathname components, inserting '/' as needed.
-                    If any component is an absolute path, all previous path components
-                    will be discarded.  An empty last part will result in a path that
-                    ends with a separator."""
-                    a = os.fspath(a)
-                    sep = _get_sep(a)
-                    path = a
-                    try:
-                        if not p:
-                            path[:0] + sep  #23780: Ensure compatible data type even if p is null.
-                        for b in map(os.fspath, p):
-                            if b.startswith(sep):
-                                path = b
-                            elif not path or path.endswith(sep):
-                                path += b
-                            else:
-                                path += sep + b
-                    except (TypeError, AttributeError, BytesWarning):
-                        genericpath._check_arg_types('join', a, *p)
-                        raise
-                    return path'''
-            )
-        else:
-            raise NotImplementedError(
-                'Version "{version}" is not supported.'.format(version=version)
-            )
+                Reformat the single paragraph in 'text' to fit in lines of no more
+                than 'width' columns, and return a new string containing the entire
+                wrapped paragraph.  As with wrap(), tabs are expanded and other
+                whitespace characters converted to space.  See TextWrapper class for
+                available keyword args to customize wrapping behaviour.
+                """
+                w = TextWrapper(width=width, **kwargs)
+                return w.fill(text)'''
+        )
 
-        content = [":func:`os.path.join`"]
+        content = [":func:`textwrap.fill`"]
         self._test_import(content, expected)  # pylint: disable=no-value-for-parameter
 
-    def test_class(self):
+
+@unittest.skipIf(
+    _skip_from_ssl_error("https://ways.readthedocs.io/en/latest/objects.inv"),
+    "URL could not be reached",
+)
+class InventoryReader(unittest.TestCase):
+    """Check that external queries to Sphinx intersphinx inventtory files work."""
+
+    @mock.patch("code_include.source_code._get_source_code_from_object")
+    @mock.patch("code_include.source_code._get_app_inventory")
+    def _test_import(
+        self,
+        content,
+        expected,
+        _get_app_inventory,
+        _get_source_code_from_object,
+    ):
+        """A generic test function. It tests for some source code from an inventory.
+
+        Args:
+            content (list[str]):
+                The lines that the user provides in a standard code-include block.
+            expected (str):
+                The converted source-code text that will be tested for.
+            _get_app_inventory (:class:`mock.mock.MagicMock`):
+                The function that gets dictionary information (which
+                later finds the Sphinx Python source-code).
+            _get_source_code_from_object (:class:`mock.mock.MagicMock`):
+                The function that's used to import a Python object to
+                get its source-code to find every HTML file-path and
+                tag-able header.
+
+        """
+        _get_source_code_from_object.return_value = ""
+        path = "https://ways.readthedocs.io/en/latest/objects.inv"
+        _get_app_inventory.return_value = common.load_cache_from_url(path)
+        _get_source_code_from_object.return_value = ""
+
+        directive = common.make_mock_directive(content)
+        nodes = directive.run()
+
+        self.assertNotEqual([], nodes)
+        self.assertEqual(1, len(nodes))
+        self.assertEqual(expected, nodes[0].astext())
+
+    @mock.patch("code_include.source_code._get_source_module_data")
+    def test_class(self, _get_source_module_data):
         """Get the source-code of an importable class."""
-        content = [":class:`ways.plugin.Plugin`"]
+        _get_source_module_data.return_value = (
+            "https://ways.readthedocs.io/en/latest/_modules/ways/base/plugin.html",
+            "Plugin",
+        )
+
+        content = [":class:`ways.base.plugin.Plugin`"]
         expected = textwrap.dedent(
             """\
+            @six.add_metaclass(PluginRegistry)
             class Plugin(object):
 
                 '''An add-on that is later retrieved by Context to gather its data.'''
@@ -337,10 +327,6 @@ class Reader(unittest.TestCase):
                 add_to_registry = True
                 _data = dict()
 
-                def __init__(self):
-                    '''Create the object and keep a reference to the cache.'''
-                    super(Plugin, self).__init__()
-
                 @property
                 def data(self):
                     '''dict[str]: The display properties (like {'color': 'red'}).'''
@@ -348,17 +334,30 @@ class Reader(unittest.TestCase):
 
                 @data.setter
                 def data(self, value):
+                    '''Set the data on this instance with whatever value is.
+
+                    Args:
+                        value (dict[str]): The new values for this instance.
+
+                    '''
                     self._data = value"""
         )
 
         self._test_import(content, expected)  # pylint: disable=no-value-for-parameter
 
-    def test_function(self):
+    @mock.patch("code_include.source_code._get_source_module_data")
+    def test_function(self, _get_source_module_data):
         """Get the source-code of an importable function."""
-        content = [":func:`ways.plugin.get_assignment`"]
+        _get_source_module_data.return_value = (
+            "https://ways.readthedocs.io/en/latest/_modules/ways/base/plugin.html",
+            "get_assignment",
+        )
+
+        content = [":func:`ways.base.plugin.get_assignment`"]
         expected = textwrap.dedent(
             """\
             def get_assignment(obj):
+                '''str: Get an object's assignment or fallback to ways.DEFAULT_ASSIGNMENT.'''
                 try:
                     return obj.get_assignment()
                 except AttributeError:
@@ -367,44 +366,71 @@ class Reader(unittest.TestCase):
 
         self._test_import(content, expected)  # pylint: disable=no-value-for-parameter
 
-    def test_method(self):
+    @mock.patch("code_include.source_code._get_source_module_data")
+    def test_method(self, _get_source_module_data):
         """Get the source-code of an importable method."""
-        content = [":meth:`ways.plugin.Plugin.__init__`"]
+        _get_source_module_data.return_value = (
+            "https://ways.readthedocs.io/en/latest/_modules/ways/base/plugin.html",
+            "DataPlugin.get_groups",
+        )
+
+        content = [":meth:`ways.base.plugin.DataPlugin.get_groups`"]
         expected = textwrap.dedent(
             """\
-            def __init__(self):
-                '''Create the object and keep a reference to the cache.'''
-                super(Plugin, self).__init__()"""
+            def get_groups(self):
+                '''Get the groups that this Plugin evaluates onto.
+
+                Note:
+                    The term 'groups' is not the same as the assignment of a Plugin.
+                    They are two different things.
+
+                Returns:
+                    tuple[str]: The groups.
+
+                '''
+                value = check.force_itertype(self._info.get('groups', ('*', )), itertype=tuple)
+                is_empty = not [val for val in value if val.strip()]
+                if is_empty:
+                    value = ('*', )
+                return value"""
         )
 
         self._test_import(content, expected)  # pylint: disable=no-value-for-parameter
 
-    def test_module(self):
+    @mock.patch("code_include.source_code._get_source_module_data")
+    def test_module(self, _get_source_module_data):
         """Get the source-code of an importable module."""
-        content = [":meth:`ways.plugin`"]
+        _get_source_module_data.return_value = (
+            "https://ways.readthedocs.io/en/latest/_modules/ways/base/plugin.html",
+            "",
+        )
+
+        content = [":mod:`ways.base.plugin`"]
+
         expected = textwrap.dedent(
             """\
             #!/usr/bin/env python
             # -*- coding: utf-8 -*-
 
-            '''A module that holds Plugin classes - objects that combine into a Context.'''
+            '''A module that holds Plugin classes and objects that combine into a Context.'''
 
             # IMPORT STANDARD LIBRARIES
             import uuid
 
             # IMPORT THIRD-PARTY LIBRARIES
-            import ways
             import six
 
+            # IMPORT WAYS LIBRARIES
+            import ways
+
             # IMPORT LOCAL LIBRARIES
-            from .core import check
-            from . import common
-            from . import cache
+            from ..core import check
+            from ..helper import common
 
 
             class PluginRegistry(type):
 
-                '''A metaclass that adds newly-created Plugin objects to a cache.'''
+                '''A metaclass that adds new Plugin objects to a cache.'''
 
                 def __new__(mcs, clsname, bases, attrs):
                     '''Add the created object to the HistoryCache.'''
@@ -429,6 +455,7 @@ class Reader(unittest.TestCase):
                     return new_class
 
 
+            # pylint: disable=too-few-public-methods
             @six.add_metaclass(PluginRegistry)
             class Plugin(object):
 
@@ -437,10 +464,6 @@ class Reader(unittest.TestCase):
                 add_to_registry = True
                 _data = dict()
 
-                def __init__(self):
-                    '''Create the object and keep a reference to the cache.'''
-                    super(Plugin, self).__init__()
-
                 @property
                 def data(self):
                     '''dict[str]: The display properties (like {'color': 'red'}).'''
@@ -448,6 +471,12 @@ class Reader(unittest.TestCase):
 
                 @data.setter
                 def data(self, value):
+                    '''Set the data on this instance with whatever value is.
+
+                    Args:
+                        value (dict[str]): The new values for this instance.
+
+                    '''
                     self._data = value
 
 
@@ -468,7 +497,7 @@ class Reader(unittest.TestCase):
 
                 add_to_registry = False
 
-                def __init__(self, sources, info, assignment):
+                def __init__(self, name, sources, info, assignment):
                     '''Create the object and set its sources.
 
                     Args:
@@ -505,6 +534,7 @@ class Reader(unittest.TestCase):
                     except AttributeError:
                         pass
 
+                    self.name = name
                     self._info = info
                     self.sources = tuple(sources)
                     self._data = self._info.get('data', dict())
@@ -512,45 +542,29 @@ class Reader(unittest.TestCase):
 
                     super(DataPlugin, self).__init__()
 
-                def _toggle_read_only(self):
-                    '''Change the stored data to be overridable or not.'''
-                    try:
-                        self._info.settable = not self._info.settable
-                    except AttributeError:
-                        pass
-
                 @classmethod
                 def _required_keys(cls):
                     '''tuple[str]: Keys that must be set in our Plugin.'''
                     return ('hierarchy', )
 
-                def is_findable(self):
-                    '''If this Plugin is okay to query directly.
+                def is_path(self):
+                    '''If the mapping is a filepath or None if unsure.
 
-                    Some plugins are meant to be combined with other plugins and aren't
-                    actually meant to be queried by themselves. This kind of behavior is
-                    allowed by our system using the 'findable' key.
+                    Returns:
+                        bool or NoneType: If the mapping is a path to a file/folder on disk.
 
                     '''
-                    return self._info.get('findable', True)
-
-                def is_hidden(self):
-                    '''bool: If this Plugin would be hidden (from view or from code).'''
-                    return self._info.get('hidden', False)
-
-                def is_navigatable(self):
-                    '''bool: If we're allowed to move into the mapping in this instance.'''
-                    return self._info.get('navigatable', True)
-
-                def is_selectable(self):
-                    '''bool: If this mapping were displayed - whether it is selectable.'''
-                    return self._info.get('selectable', True)
+                    try:
+                        return self._info['path']
+                    except KeyError:
+                        return None
 
                 def get_assignment(self):
+                    '''str: Where this Plugin lives in Ways, along with its hierarchy.'''
                     return self.assignment
 
                 def get_groups(self):
-                    '''The groups that this Plugin evaluates onto.
+                    '''Get the groups that this Plugin evaluates onto.
 
                     Note:
                         The term 'groups' is not the same as the assignment of a Plugin.
@@ -570,10 +584,6 @@ class Reader(unittest.TestCase):
                     '''tuple[str] or str: The location that this Plugin exists within.'''
                     return self._info['hierarchy']
 
-                def get_id(self):
-                    '''str: The name associated with this Plugin.'''
-                    return self._info['id']
-
                 def get_mapping(self):
                     '''str: The physical location of this Plugin (on the filesystem).'''
                     try:
@@ -587,11 +597,12 @@ class Reader(unittest.TestCase):
 
                 def get_max_folder(self):
                     '''str: The furthest location up that this plugin can navigate to.'''
-                    return self._info.get('max_folder', self.get_mapping())
+                    return self._info.get('max_folder', '')
 
                 def get_platforms(self):
-                    '''tuple[str]: The platforms that this Plugin is allowed to run on.'''
-                    return self._info.get('platforms', ('*', ))
+                    '''set[str]: The platforms that this Plugin is allowed to run on.'''
+                    platforms = ways.get_known_platfoms()
+                    return set(self._info.get('platforms', platforms))
 
                 def get_uses(self):
                     '''tuple[str]: The Context hierarchies this instance depends on.'''
@@ -599,7 +610,7 @@ class Reader(unittest.TestCase):
 
                 def get_uuid(self):
                     '''str: A unique ID for this plugin.'''
-                    return self._info['uuid']
+                    return self._info.get('uuid', '')
 
                 def __repr__(self):
                     '''str: The information needed to reproduce this instance.'''
@@ -608,28 +619,20 @@ class Reader(unittest.TestCase):
                         sources=self.sources,
                         data=dict(self._info))
 
-                    # return '{cls_}(sources={sources!r}, uuid={uid})'.format(
-                    #     cls_=self.__class__.__name__,
-                    #     sources=self.sources,
-                    #     uid=self.get_uuid())
-
-                # def __str__(self):
-                #     '''str: A pretty-print of the plugin.'''
-                #     return '{cls_}(sources={sources!r}, uuid={uid})'.format(
-                #         cls_=self.__class__.__name__,
-                #         sources=self.sources,
-                #         uid=self.get_uuid())
+                def __str__(self):
+                    '''str: A more concise print-out of this instance.'''
+                    return '{cls_}(hierarchy={hierarchy}, sources={sources!r})'.format(
+                        cls_=self.__class__.__name__,
+                        hierarchy=self.get_hierarchy(),
+                        sources=self.sources)
 
 
             def get_assignment(obj):
+                '''str: Get an object's assignment or fallback to ways.DEFAULT_ASSIGNMENT.'''
                 try:
                     return obj.get_assignment()
                 except AttributeError:
-                    return common.DEFAULT_ASSIGNMENT
-
-
-            if __name__ == '__main__':
-                print(__doc__)"""
+                    return common.DEFAULT_ASSIGNMENT"""
         )
 
         self._test_import(content, expected)  # pylint: disable=no-value-for-parameter
